import { SeleccionAlbaran } from 'dialogo-seleccion-albaran';
import { DatosTransporte } from 'dialogo-datos-transporte';
import { Boton } from 'boton';
import { Campo } from 'campo';
import { Bloque } from 'bloque';
import { Tabla } from 'tabla';
import { Cortina } from 'cortina';
import { Albaran, ExistenciasLote, LineaPedidoPreparado, PedidoPreparado, RespuestaResumenBultos } from 'expedicion-tipos';
declare type LineaPedido = {
    N_LIN: number;
    GAART_COD: string;
    DESCRIP: string;
    GAART_DESCRIP: string;
    GAALMNOM_COD: string;
    C_PDTE: number;
    C_PDTE2: number;
    GAART_LOTES_SN: 'S' | 'N';
    GAART_UBIC_SN: 'S' | 'N';
    Existencias: ExistenciasLote[];
};
declare type Pedido = {
    GGPEDTIP_COD: string;
    N_PEDIDO: number;
    OBSER: string;
    GGMENVIO_COD: string;
    GGMENVIO_DESCRIP: string;
    GGVEHICULO_COD: string;
    GGVEHICULO_DESCRIP: string;
    GGCLIPR_COD: number;
    GGCLIPR_NOMBRE: string;
    MostrarAlbaranValorado: boolean;
    AlbaranValorado: boolean;
    Albaranes: Albaran[];
    Lineas: LineaPedido[];
    BultosPorLinea: boolean;
    DatosEnvioInicial: boolean;
};
declare type DisponibilidadPorArticulo = {
    [GAART_COD: string]: ExistenciasLote[];
};
export declare class PantallaExpedicion {
    protected botonNuevo: Boton;
    protected botonConfirmar: Boton;
    protected botonPendiente: Boton;
    protected tablaLineas: Tabla;
    protected campoLecturaPedido: Campo;
    protected campoGGCLIPR_COD: Campo;
    protected campoGGCLIPR_NOMBRE: Campo;
    protected campoReferencia: Campo;
    protected bloqueReferencia: Bloque;
    protected campoC_PREP: Campo;
    protected cortina: Cortina;
    protected lecturaPedido: string;
    protected lecturaArticulo: string;
    protected lecturaCantidad: number;
    protected pedido: Pedido;
    protected pedidoPreparado: PedidoPreparado;
    protected disponible: DisponibilidadPorArticulo;
    protected lineaSeleccionada: LineaPedidoPreparado;
    constructor();
    protected inicializaComponentes(): {
        botonNuevo: import("boton").DefinicionBoton;
        botonConfirmar: import("boton").DefinicionBoton;
        botonPendiente: import("boton").DefinicionBoton;
        tablaLineas: any;
        campoLecturaPedido: import("campo").DefinicionCampo;
        campoGGCLIPR_COD: import("campo").DefinicionCampo;
        campoGGCLIPR_NOMBRE: import("campo").DefinicionCampo;
        campoReferencia: import("campo").DefinicionCampo;
        bloqueReferencia: import("bloque").DefinicionBloque;
        campoC_PREP: import("campo").DefinicionCampo;
    };
    protected generaCortina(): import("cortina").DefinicionCortina;
    actualizaVista(): void;
    protected nuevoPedido(): Promise<void>;
    protected actualizaModelo(): void;
    protected inicializa(): void;
    protected pideDatosTransporte(p_datosTransporte: DatosTransporte): Promise<DatosTransporte>;
    protected cargaPedido(): Promise<boolean>;
    protected pideAlbaran(p_albaranesPedido?: Albaran[]): Promise<SeleccionAlbaran>;
    protected seleccionaDatosTransporte(p_resultadoDatosTransporte: DatosTransporte): void;
    protected seleccionaAlbaran(p_resultadoSeleccionAlbaran: SeleccionAlbaran): void;
    protected actualizaPreparadoArticuloSinLotesNiUbicaciones(p_valor: any): Promise<void>;
    protected pideYActualizaBultos(p_linea: any, p_disponible: ExistenciasLote[], p_preparado: any): Promise<void>;
    protected actualizaLinea(p_linea: any, p_disponible: any, p_preparado: any): void;
    protected seleccionaLinea(p_lineaSeleccionada: any): Promise<void>;
    protected sePuedeConfirmarPedido(): boolean;
    protected actualizaPreparadoConResumenBultos(p_resumenBultos: any, p_opciones: any): void;
    protected confirmaPedido(p_opciones: {
        dejarPendiente: boolean;
    }): Promise<void>;
    protected generaResumenBultos(): Promise<RespuestaResumenBultos>;
    protected pideResumenBultos(p_resumenBultos: RespuestaResumenBultos, p_dejarPendiente: boolean): Promise<unknown>;
    protected generaAlbaran(): Promise<{
        GGALBTIP_COD: string;
        N_ALBARAN: number;
        PREGUNTA_HIST: string;
    }>;
    protected obtenerLinea(): Promise<void>;
}
export {};
