﻿define('pantalla-actualiza-albaran',
    [
        'api',
        'componentes',
        'dialogo-resumen-bultos',
        'utilidades'
    ],
    function (
        api,
        componentes,
        DialogoResumenBultos,
        utilidades
    ) {
        let
            componente = componentes.componente,
            mensaje = componentes.mensaje,
            modal = componentes.modal,
            dialogo = componentes.dialogo,
            interpola = utilidades.interpola;

        class ModeloLinea {
            constructor(p_gaart_cod, p_gaart_descrip, p_c_uni1, p_c_uni2) {
                this.gaart_cod = p_gaart_cod;
                this.gaart_descrip = p_gaart_descrip;
                this.c_uni1 = p_c_uni1;
                this.c_uni2 = p_c_uni2;
            }
        }

        class Modelo {
            constructor() {
                this.albaran = null;
                this.ggalbtip_cod = null;
                this.n_albaran = null;
                this.fecha = null;
                this.ggclipr_nom = null;
                this.Parametros = {};
                this.lineas = [];

                this._botonNuevo = componente('nuevo')
                    .clic(this.limpia.bind(this))
                    .crea();

                this._botonActivar = componente('activar')
                    .clic(this.activaAlbaran.bind(this))
                    .crea();

                this._inputAlbaran = componente('albaran')
                    .cambio(this.cargaAlbaran.bind(this))
                    .crea();

                this._inputFecha = componente('fecha')
                    .crea();

                this._inputCliente = componente('cliente')
                    .crea();

                this._tabla = componente('lineas')
                    .columnas([
                        { propiedad: 'gaart_cod', titulo: 'Artículo', ancho: '6em', orden: 'asc' },
                        { propiedad: 'gaart_descrip', titulo: 'Descripción', orden: 'asc' },
                        { propiedad: 'c_uni1', titulo: 'Cantidad', ancho: '4em', numerico: true }
                    ])
                    .crea();

                this.actualizaVista();
            }

            actualizaVista() {
                this._inputAlbaran.valor(this.albaran);
                this._inputFecha.valor(this.fecha);
                this._inputCliente.valor(this.ggclipr_nom);

                this._tabla.datos(this.lineas);
            }

            limpia() {
                this.albaran = null;
                this.ggalbtip_cod = null;
                this.n_albaran = null;
                this.fecha = null;
                this.ggclipr_nom = null;
                this.lineas = [];
                this.actualizaVista();

                this._inputAlbaran.foco();
            }

            actualizaModelo() {
                this.albaran = this._inputAlbaran.valor().toUpperCase();
            }

            async cargaAlbaran() {
                this.actualizaModelo();

                let v_albaran;
                try {
                    [v_albaran] = await api('/Expedicion/AlbaranPendiente')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem('EMPRGRUP_COD'),
                            emp_cod: sessionStorage.getItem('EMPR_COD'),
                            lecturaalbaran: this.albaran
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje(v_mensaje).error().muestra();
                    this.limpia();
                    return;
                }

                this.ggalbtip_cod = v_albaran.GGALBTIP_COD;
                this.n_albaran = v_albaran.N_ALBARAN;
                this.fecha = v_albaran.FECHA;
                this.ggclipr_nom = v_albaran.GGCLIPR_NOM;

                this.lineas = v_albaran.Lineas.map(function (p_linea) {
                    return new ModeloLinea(p_linea.GAART_COD, p_linea.DESCRIP, p_linea.C_UNI1, p_linea.C_UNI2);
                });
                this.actualizaVista();
            };

            async activaAlbaran() {
                this.actualizaModelo();

                let v_resumenBultos;
                try {
                    [v_resumenBultos] = await api('/Expedicion/ResumenBultosAlbaranPendiente')
                        .metodo('PUT')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem('EMPRGRUP_COD'),
                            emp_cod: sessionStorage.getItem('EMPR_COD'),
                            ggalbtip_cod: this.ggalbtip_cod,
                            n_albaran: this.n_albaran,
                            lineas: this.lineas
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje(v_mensaje).error().muestra();
                    this.limpia();
                    return;
                }

                let v_resultadoPideResumenBultos = await this.pideResumenBultos(v_resumenBultos);
                if (typeof v_resultadoPideResumenBultos === 'undefined')
                    return;

                let v_opcion;
                [v_resumenBultos, v_opcion] = v_resultadoPideResumenBultos;

                let v_resultado = await mensaje(
                    interpola('¿Desea cambiar el estado del albarán de Pendiente a Activo y cambiar la fecha del {v_fechaActual} al {v_nuevaFecha}?', {
                        v_fechaActual: this.fecha,
                        v_nuevaFecha: moment(new Date()).format("DD/MM/YY")
                    }))
                    .pregunta(['Si', 'No']);
                if (typeof v_resultado === 'undefined')
                    return;

                let [v_respuesta] = v_resultado;
                if (v_respuesta === 1 /* NO */)
                    return;

                async function intentaCambiarEstadoAlbaran() {
                    let v_cambiaEstadoAlbaran;
                    try {
                        [v_cambiaEstadoAlbaran] = await this.cambiaEstadoAlbaran(v_resumenBultos, v_opcion === 'confirmaEImprime');
                    }
                    catch ([v_mensaje]) {
                        await mensaje(v_mensaje).error().muestra();
                        this.limpia();
                        return;
                    }

                    this.Parametros = v_cambiaEstadoAlbaran.Parametros;

                    if (v_cambiaEstadoAlbaran.Parametros && v_cambiaEstadoAlbaran.Parametros.P_PREGUNTA) {
                        let v_resultadoDialogoPregunta = await mensaje(v_cambiaEstadoAlbaran.Parametros.P_PREGUNTA)
                            .pregunta(['Si', 'No']);

                        if (typeof v_resultadoDialogoPregunta === 'undefined')
                            return;

                        let [v_respuestaPregunta] = v_resultadoDialogoPregunta;
                        if (v_respuestaPregunta === 0 /* Si */) {
                            this.Parametros.P_RESPUESTA = 'S';
                        } else {
                            this.Parametros.P_RESPUESTA = 'N';
                        }

                        intentaCambiarEstadoAlbaran.apply(this);
                    }
                    else {
                        mensaje('Cambio realizado').notificacion().muestra();
                        this.limpia();
                    }
                }

                intentaCambiarEstadoAlbaran.apply(this);
            };

            async cambiaEstadoAlbaran(p_resumenBultos, p_imprimeEtiquetas) {
                let v_resultado = await api('/Expedicion/CambiarEstadoAlbaran')
                    .metodo('POST')
                    .parametros({
                        empgrupo_cod: sessionStorage.getItem('EMPRGRUP_COD'),
                        emp_cod: sessionStorage.getItem('EMPR_COD'),
                        ggalbtip_cod: this.ggalbtip_cod,
                        n_albaran: this.n_albaran,
                        ggmenvio_cod: p_resumenBultos.GGMENVIO_COD,
                        ggvehiculo_cod: p_resumenBultos.GGVEHICULO_COD,
                        n_bultos: p_resumenBultos.N_BULTOS,
                        peso: p_resumenBultos.PESO,
                        n_etiquetas: p_resumenBultos.N_ETIQUETAS,
                        OpcionEtiquetas: p_resumenBultos.OpcionPredeterminada,
                        ImprimirEtiquetas: p_imprimeEtiquetas,
                        ImprimirValorado: p_resumenBultos.AlbaranValorado,
                        Parametros: this.Parametros,
                    })
                    .invoca();

                return v_resultado;
            }

            async pideResumenBultos(p_resumenBultos) {
                //return await modal('dialogo-resumen-bultos')
                //    .construye(p_modal => {
                //        let v_dialogo = new DialogoResumenBultos(p_modal, p_resumenBultos);
                //        v_dialogo.actualizaVista();
                //        return v_dialogo;
                //    })
                //    .abre();

                return await dialogo()
                    .titulo('Datos de envío')
                    .plantilla('dialogo-resumen-bultos')
                    .construye(p_modal =>
                        new DialogoResumenBultos(p_modal,
                            Object.assign(p_resumenBultos)))
                    .abre();
            }
        }

        return Modelo;
    });