﻿define('--plantilla',
    [
        'utilidades'
    ],
    function (
        utilidades
    ) {
        let
            interpola = utilidades.interpola,
            traza = utilidades.traza;

        class Renderizado {
            constructor(p_documento) {
                this._documento = p_documento;
            }

            documento() {
                return this._documento;
            }

            contenido() {
                return this._documento.body.firstElementChild;
            }

            renombra(p_idPrefijo, p_elementos) {
                for (var v_indice = 0; v_indice < p_elementos.length; v_indice++) {
                    let v_elemento = p_elementos[v_indice];
                    let v_nuevoId;
                    if (v_elemento.id.length > 0)
                        v_nuevoId = `${p_idPrefijo}.${v_elemento.id}`;
                    else
                        v_nuevoId = p_idPrefijo;
                    //traza(`${v_elemento.id} => ${v_nuevoId}`);
                    v_elemento.id = v_nuevoId;
                }
            }

            corrigeAtributoFor(p_idPrefijo, p_etiquetas) {
                for (var v_indice = 0; v_indice < p_etiquetas.length; v_indice++) {
                    let v_etiqueta = p_etiquetas[v_indice];
                    let v_nuevoFor;
                    if (v_etiqueta.htmlFor.length > 0)
                        v_nuevoFor = `${p_idPrefijo}.${v_etiqueta.htmlFor}`;
                    else
                        v_nuevoFor = p_idPrefijo;
                    //traza(`${v_etiqueta.htmlFor} => ${v_nuevoFor}`);
                    v_etiqueta.htmlFor = v_nuevoFor;
                }
            }

            inserta(p_opciones) {
                let { contenedor: v_contenedor, prefijoId: v_prefijoId } = p_opciones || {};
                if (typeof v_contenedor === 'undefined')
                    v_contenedor = document.body;

                if (typeof v_prefijoId !== 'undefined') {
                    let v_elementosConId = Array.from(this._documento.querySelectorAll('*[id]'));

                    if (this._documento.body.firstElementChild !== null && this._documento.body.firstElementChild.id.length === 0)
                        v_elementosConId = [this._documento.body.firstElementChild].concat(v_elementosConId);

                    this.renombra(v_prefijoId, v_elementosConId);

                    let v_etiquetasConFor = Array.from(this._documento.querySelectorAll('*[for]'));

                    this.corrigeAtributoFor(v_prefijoId, v_etiquetasConFor);
                }

                let v_elemento = this._documento.body.firstElementChild;
                if (v_contenedor.nodeType === Node.COMMENT_NODE)
                    v_contenedor.replaceWith(v_elemento);
                else
                    v_contenedor.appendChild(v_elemento);

                this._documento = null;

                return new Insertado(v_elemento);
            }
        }

        class Insertado {
            constructor(p_elemento) {
                this._elemento = p_elemento;
            }

            elemento() {
                return this._elemento;
            }

            suprime() {
                this._elemento.remove();
            }
        }

        class DefinicionPlantilla {
            constructor(p_valores) {
                this.valores = { nombre: null, ...p_valores };
            }

            //nombre(p_nombre) {
            //    return new DefinicionPlantilla({ ...this.valores, nombre: p_nombre });
            //}

            //secciones(p_secciones) {
            //    return new DefinicionPlantilla(Object.assign(this.valores, { secciones: p_secciones }));
            //}

            //async carga(p_valores) {
            //    if (typeof this.valores.nombre !== 'string')
            //        throw new Error('No es un texto');

            //    let v_promesa = new Promise((p_resuelve, p_rechaza) =>
            //        require([`text!${this.valores.nombre}`], p_plantilla => p_resuelve(p_plantilla))
            //    );
            //    let v_texto = await v_promesa;

            //    v_texto = interpola(v_texto, p_valores);

            //    return this.crea(v_texto);
            //}

            nombre(p_nombre) {
                return new DefinicionPlantilla({ ...this.valores, nombre: p_nombre });
            }

            secciones(p_secciones) {
                return new DefinicionPlantilla(Object.assign(this.valores, { secciones: p_secciones }));
            }

            async carga(p_valores) {
                if (typeof this.valores.nombre !== 'string')
                    throw new Error('No es un texto');

                function dependencia(p_plantilla) {
                    let v_dependencia = `text!${p_plantilla.indexOf('.html') === -1 ? p_plantilla + '.html' : p_plantilla}`;
                    return v_dependencia;
                }

                // si no tiene extensión, supongo que es extensión .html
                let v_nombre = dependencia(this.valores.nombre);

                let v_nombresSeccion = Object.getOwnPropertyNames(this.valores.secciones || {});
                let v_secciones = v_nombresSeccion.reduce((p_secciones, p_nombre) => {
                    p_secciones[p_nombre] = dependencia(this.valores.secciones[p_nombre]);
                    return p_secciones;
                }, {});

                let v_dependencias = [v_nombre];
                if (typeof v_secciones !== 'undefined')
                    v_dependencias = v_dependencias.concat(v_nombresSeccion
                        .map(p_nombre => v_secciones[p_nombre])
                    );

                let v_promesa = new Promise((p_resuelve, p_rechaza) =>
                    require(v_dependencias, function () {
                        let v_plantillas = Array.from(arguments);

                        let v_textosSeccion = v_nombresSeccion.reduce((p_textosSeccion, p_nombreSeccion) => {
                            let v_plantilla = v_secciones[p_nombreSeccion];
                            let v_indice = v_dependencias.indexOf(v_plantilla);
                            let v_texto = v_plantillas[v_indice];
                            p_textosSeccion[p_nombreSeccion] = v_texto;
                            return p_textosSeccion;
                        }, {})

                        return p_resuelve({ texto: v_plantillas[0], secciones: v_textosSeccion });
                    })
                );

                let v_resultado = await v_promesa;
                let v_texto = v_resultado.texto;

                v_texto = interpola(v_texto, p_valores);

                return this.crea(v_texto, v_resultado.secciones);
            }

            crea(p_texto, p_secciones) {
                return new Plantilla(p_texto, p_secciones);
            }
        }

        class Plantilla {
            constructor(p_texto, p_secciones) {
                this._texto = p_texto;
                this._secciones = p_secciones || {};
            }

            texto() {
                return this._texto;
            }

            // genera un documento con el contenido de la plantilla
            renderiza(/*opciones*/) {
                let v_opciones = arguments.length === 1 ? arguments[0] : {};
                let { prefijoId: v_prefijoId } = v_opciones;

                if (this._texto === null)
                    throw new Error('Plantilla sin texto.');

                let v_parser = new DOMParser();
                let v_documento = v_parser.parseFromString(this._texto, 'text/html');
                if (v_documento.body.firstElementChild === null)
                    throw new Error('No se permiten plantillas vacías.');

                if (typeof this._secciones !== 'undefined') {
                    let v_iterador = v_documento.createNodeIterator(
                        v_documento.body,
                        NodeFilter.SHOW_COMMENT,
                        {
                            acceptNode: p_nodo => p_nodo.nodeValue.trimLeft().startsWith('!') ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_REJECT
                        }
                    );

                    let v_marcadores = [];
                    let v_nodo;
                    while (v_nodo = v_iterador.nextNode()) {
                        const v_json = v_nodo.nodeValue.trimLeft().slice(1);
                        let v_valor;
                        try {
                            v_valor = eval(`({${v_json}})`);
                            v_marcadores.push({ nodo: v_nodo, valor: v_valor });
                        }
                        catch {
                            // nada
                        }
                    }

                    v_marcadores.forEach(p_marcador => {
                        if (typeof p_marcador.valor.seccion !== 'string')
                            return;

                        let v_textoSeccion = this._secciones[p_marcador.valor.seccion];
                        if (typeof v_textoSeccion === 'undefined')
                            return;

                        let v_plantillaSeccion = plantilla().crea(this._secciones[p_marcador.valor.seccion]);
                        v_plantillaSeccion.inserta({ contenedor: p_marcador.nodo });
                    });
                }

                return new Renderizado(v_documento);
            }


            // genera un documento con el contenido de la plantilla y mueve el primer elemento del cuerpo a un nuevo contenedor
            inserta(/*opciones*/) {
                let v_opciones = arguments.length === 1 ? arguments[0] : {};
                let v_renderizado = this.renderiza(v_opciones);
                return v_renderizado.inserta(v_opciones);
            }
        }

        function plantilla(p_definicion) {
            let v_definicion = new DefinicionPlantilla();
            if (typeof p_definicion === 'string')
                v_definicion = v_definicion.nombre(p_definicion);
            else if (p_definicion !== null && typeof p_definicion === 'object') {
                if (typeof p_definicion.nombre === 'string')
                    v_definicion = v_definicion.nombre(p_definicion.nombre);
                if (p_definicion.secciones !== null && typeof p_definicion.secciones === 'object')
                    v_definicion = v_definicion.secciones(p_definicion.secciones);
            }
            return v_definicion;
        }

        return plantilla;
    }); 