﻿define('desplegable',
    [
        'foco',
        'utilidades'
    ],
    function (
        foco,
        utilidades
    ) {
        let interpola = utilidades.interpola;

        function desplegable(p_elemento) {
            function DefinicionDesplegable(p_elemento, p_valores) {

                this.valores = function (p_valoresDesplegable) {
                    return new DefinicionDesplegable(p_elemento, Object.assign(p_valores, { valores: p_valoresDesplegable }));
                };

                this.cambio = function (p_funcion) {
                    return new DefinicionDesplegable(p_elemento, Object.assign(p_valores, { cambio: p_funcion }));
                };

                this.crea = function () {
                    return new Desplegable(p_elemento, p_valores);
                };
            }

            function Desplegable(p_elemento, p_valores, p_botonLov) {
                let v_valoresDesplegable = p_valores.valores;

                this.valores = function (p_valoresDesplegable) {
                    v_valoresDesplegable = Array.isArray(p_valoresDesplegable) ? p_valoresDesplegable : [];
                    actualiza();
                }

                this.valor = function (p_valor) {
                    if (arguments.length === 1)
                        p_elemento.value = p_valor;
                    else
                        return p_elemento.value;
                };

                this.foco = function () {
                    foco.en(p_elemento);
                    return this;
                };

                this.deshabilita = function () {
                    p_elemento.setAttribute("disabled", "true");
                    return this;
                };

                this.habilita = function () {
                    p_elemento.removeAttribute("disabled");
                    return this;
                };

                this.habilitado = function (p_valor) {
                    if (arguments.length === 0)
                        return !p_elemento.hasAttribute('disabled');
                    else if (p_valor)
                        this.habilita();
                    else
                        this.deshabilita();
                };

                this.visible = function (p_valor) {
                    if (arguments.length === 0)
                        return !p_elemento.classList.contains('oculto');
                    else {
                        let v_elementos = [p_elemento].concat(Array.from(p_elemento.labels));
                        if (p_valor)
                            v_elementos.forEach(p => p.classList.remove('oculto'));
                        else
                            v_elementos.forEach(p => p.classList.add('oculto'));
                    }
                };

                p_elemento.setAttribute('autocomplete', 'off');

                if (typeof p_valores.cambio === 'function')
                    p_elemento.addEventListener('input', () => p_valores.cambio(this.valor()));

                //p_elemento.addEventListener('focus', () => p_elemento.select());

                function actualiza() {
                    // primero elimino todos los hijos que hubiera
                    while (p_elemento.firstChild)
                        p_elemento.removeChild(p_elemento.firstChild);

                    // por cada valor genero un elemento <option>
                    for (let v_indice = 0; v_indice < v_valoresDesplegable.length; v_indice++) {
                        let v_opcion = document.createElement('option');
                        v_opcion.text = v_valoresDesplegable[v_indice];
                        p_elemento.appendChild(v_opcion);
                    }
                }

                if (Array.isArray(v_valoresDesplegable))
                    actualiza();
            }

            let v_elemento = p_elemento instanceof HTMLElement ? p_elemento : document.getElementById(p_elemento);

            return new DefinicionDesplegable(v_elemento, {
                cambio: void 0
            });
        }

        return desplegable;
    });