﻿define('pregunta',
    [
        '--plantilla',
        'dialogo',
        'label',
        'boton',
        'bloque',
        'utilidades'
    ],
    function (
        __plantilla,
        dialogo,
        label,
        boton,
        bloque,
        utilidades
    ) {

        const
            interpola = utilidades.interpola,
            generaSecuencia = utilidades.generaSecuencia;

        function pregunta(p_texto) {
            if (arguments.length === 0) {
                p_texto = '';
            }

            class DialogoPregunta {
                constructor(p_modal, p_texto, p_error, p_opciones, p_opcionPredeterminada, p_opcionCancelar, p_elementosBotones) {
                    this._etiquetaTexto = p_modal.etiqueta('texto').valor(p_texto).crea();
                    this._botones = p_opciones.map((p_opcion, p_indice) => {
                        let v_boton = boton(p_elementosBotones[p_indice].id)
                            .clic(() => p_modal.cierra([p_indice, p_opcion]))
                        switch (p_opcion) {
                            case p_opcionPredeterminada:
                                v_boton = v_boton.tecla('Enter');
                                break;
                            case p_opcionCancelar:
                                v_boton = v_boton.tecla('Escape');
                                break;
                        }
                        v_boton = v_boton.crea();
                        return v_boton;
                    });
                }

                actualizaVista() {
                }
            }

            function Pregunta(p_texto, p_error, p_opciones, p_opcionPredeterminada, p_opcionCancelar, p_cancelable) {
                let v_texto = p_texto;
                let v_error = p_error;
                let v_opciones = p_opciones;
                let v_opcionPredeterminada = p_opcionPredeterminada;
                let v_opcionCancelar = p_opcionCancelar;
                let v_cancelable = p_cancelable;

                this.texto = function (p_texto) {
                    if (typeof p_texto !== 'string')
                        throw new Error('No es un texto válido');
                    return new Pregunta(p_texto, v_error, v_opciones, v_opcionPredeterminada, v_opcionCancelar, v_cancelable);
                };

                this.error = function () {
                    return new Pregunta(v_texto, true, v_opciones, v_opcionPredeterminada, v_opcionCancelar, v_cancelable);
                };

                this.opciones = function (p_opciones) {
                    return new Pregunta(v_texto, v_error, p_opciones, v_opcionPredeterminada, v_opcionCancelar, v_cancelable);
                }

                this.opcionPredeterminada = function (p_opcion) {
                    return new Pregunta(v_texto, v_error, p_opciones, p_opcion, v_opcionCancelar, v_cancelable)
                }

                this.opcionCancelar = function (p_opcion) {
                    return new Pregunta(v_texto, v_error, p_opciones, v_opcionPredeterminada, p_opcion, v_cancelable)
                }


                this.cancelable = function (p_cancelable) {
                    return new Pregunta(v_texto, v_error, p_opciones, p_opcionPredeterminada, v_opcionCancelar, p_cancelable)
                }

                this.muestra = async function (p_texto) {
                    if (typeof p_texto === 'string')
                        v_texto = p_texto;

                    const v_nombrePlantilla = v_error ? 'pregunta-error.pp.html' : 'pregunta-info.pp.html';

                    const [v_contenido, v_elementosBotones] = await Promise
                        .all([
                            __plantilla().nombre(v_nombrePlantilla).carga(),
                            __plantilla().nombre('boton-pregunta.pp.html').carga()]
                        )
                        .then(([p_plantillaPregunta, p_plantillaBoton]) => {
                            let v_renderizado = p_plantillaPregunta.renderiza();
                            let v_elementoOpciones = v_renderizado.documento().getElementById('opciones');
                            if (v_opciones.length > 1) 
                                v_elementoOpciones.classList.add('varias-opciones');
                            let v_elementosBotones = v_opciones.map((p_opcion, p_indice) => {
                                const v_id = `boton-${p_indice}`,
                                const v_htmlBoton = interpola(p_plantillaBoton.texto(), { id: v_id, texto: p_opcion });
                                const v_elementoBoton = __plantilla().crea(v_htmlBoton).inserta({ contenedor: v_elementoOpciones }).elemento();
                                return v_elementoBoton;
                            });
                            return [v_renderizado.contenido(), v_elementosBotones];
                        });

                    
                    const v_dialogo = dialogo()
                        .titulo(v_error ? 'Error ': 'Info')
                        .elemento(v_contenido)
                        .construye(p_modal => new DialogoPregunta(p_modal, v_texto, v_error, v_opciones, v_opcionPredeterminada, v_cancelable ? v_opcionCancelar : null, v_elementosBotones))
                        .cancelable(v_cancelable);
                    if (v_error)
                        v_dialogo = v_dialogo.estilos({ titulo: 'error' });

                    const v_resultadoPregunta = await v_dialogo.abre();
                    return v_resultadoPregunta;
                };
            }

            return new Pregunta(p_texto, false, ['Si', 'No'], 'Si', null, true);
        }

        return pregunta;
    });