﻿// @ts-nocheck
// lodash
define('dialogo-datos-transporte',
    [
        'api',
        'componentes'
    ],
    function (
        api,
        componentes
    ) {

        let
            lov = componentes.lov,
            mensaje = componentes.mensaje;

        class DialogoDatosTransporte {
            constructor(p_modal, p_datosTransporte) {
                this.modal = p_modal;

                this.datosTransporte = _.cloneDeep(p_datosTransporte);

                this._botonAceptar = p_modal.componente('aceptar')
                    .clic(this.acepta.bind(this))
                    .tecla('Enter')
                    .crea();

                this._inputGgmenvio_cod = p_modal.componente('ggmenvio_cod')
                    .valida(this.validaGgmenvio_cod.bind(this))
                    .lov(this.pideGgmenvio_cod.bind(this))
                    .crea();
                this._inputGgmenvio_descrip = p_modal.componente('ggmenvio_descrip')
                    .crea();
                this._botonBusquedaGgmenvio_cod = p_modal.boton('busqueda-ggmenvio_cod')
                    .clic(this.pideGgmenvio_cod.bind(this))
                    .crea();

                this._inputGgvehiculo_cod = p_modal.componente('ggvehiculo_cod')
                    .valida(this.validaGgvehiculo_cod.bind(this))
                    .lov(this.pideGgvehiculo_cod.bind(this))
                    .crea();
                this._inputGgvehiculo_descrip = p_modal.componente('ggvehiculo_descrip')
                    .crea();
                this._botonBusquedaGgvehiculo_cod = p_modal.boton('busqueda-ggvehiculo_cod')
                    .clic(this.pideGgvehiculo_cod.bind(this))
                    .crea();
            }

            actualizaVista() {
                this._inputGgmenvio_cod.valor(this.datosTransporte.GGMENVIO_COD);
                this._inputGgmenvio_descrip.valor(this.datosTransporte.GGMENVIO_DESCRIP);
                this._inputGgvehiculo_cod.valor(this.datosTransporte.GGVEHICULO_COD);
                this._inputGgvehiculo_descrip.valor(this.datosTransporte.GGVEHICULO_DESCRIP);
            }

            actualizaModelo() {
                this.datosTransporte.GGMENVIO_COD = this._inputGgmenvio_cod.valor().toUpperCase();
                this.datosTransporte.GGMENVIO_DESCRIP = this._inputGgmenvio_descrip.valor().toUpperCase();

                this.datosTransporte.GGVEHICULO_COD = this._inputGgvehiculo_cod.valor().toUpperCase();
                this.datosTransporte.GGVEHICULO_DESCRIP = this._inputGgvehiculo_descrip.valor().toUpperCase();
            };

            acepta() {
                this.actualizaModelo();
                this.modal.cierra(this.datosTransporte);
            }

            async validaGgmenvio_cod({ valorHaCambiado: p_valorHaCambiado }) {
                this.actualizaModelo();

                if (!p_valorHaCambiado)
                    return;

                var v_modoEnvio;
                try {
                    [v_modoEnvio] = await api('/Expedicion/ModoEnvio')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            ggmenvio_cod: this.datosTransporte.GGMENVIO_COD
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje().error().muestra(v_mensaje);
                    return;
                }

                this.datosTransporte.GGMENVIO_COD = v_modoEnvio.COD;
                this.datosTransporte.GGMENVIO_DESCRIP = v_modoEnvio.DESCRIP;
                this.datosTransporte.GGVEHICULO_COD = this.datosTransporte.GGVEHICULO_DESCRIP = null;

                this.actualizaVista();
            }

            async pideGgmenvio_cod() {
                let v_modosEnvio;
                try {
                    [v_modosEnvio] = await api('/Expedicion/ModosEnvio')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD")
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje().error().muestra(v_mensaje);
                    return;
                }

                let v_elegido = await lov()
                    .titulo('Seleccione transportista')
                    .columnas([
                        { propiedad: 'COD', titulo: 'Código', ancho: '6em' },
                        { propiedad: 'DESCRIP', titulo: 'Nombre' }
                    ])
                    .elige(v_modosEnvio);

                if (typeof v_elegido !== 'undefined') {
                    this.datosTransporte.GGMENVIO_COD = v_elegido.COD;
                    this.datosTransporte.GGMENVIO_DESCRIP = v_elegido.DESCRIP;
                    this.datosTransporte.GGVEHICULO_COD = this.datosTransporte.GGVEHICULO_DESCRIP = null;
                    this.actualizaVista();
                }
            }

            async validaGgvehiculo_cod({ valorHaCambiado: p_valorHaCambiado }) {
                this.actualizaModelo();

                if (!p_valorHaCambiado)
                    return;

                var v_vehiculo;
                try {
                    [v_vehiculo] = await api('/Expedicion/Vehiculo')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            ggmenvio_cod: this.datosTransporte.GGMENVIO_COD,
                            cod: this.datosTransporte.GGVEHICULO_COD
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje().error().muestra(v_mensaje);
                    return;
                }

                this.datosTransporte.GGVEHICULO_COD = v_vehiculo.COD;
                this.datosTransporte.GGVEHICULO_DESCRIP = v_vehiculo.DESCRIP;

                this.actualizaVista();
            }

            async pideGgvehiculo_cod() {
                let v_vehiculos;
                try {

                    [v_vehiculos] = await api('/Expedicion/Vehiculos')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            ggmenvio_cod: this.datosTransporte.GGMENVIO_COD
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje().error().muestra(v_mensaje);
                    return;
                }

                let v_elegido = await lov()
                    .titulo('Seleccione vehículo')
                    .columnas([
                        { propiedad: 'COD', titulo: 'Código', ancho: '6em' },
                        { propiedad: 'DESCRIP', titulo: 'Nombre' }
                    ])
                    .elige(v_vehiculos);

                if (typeof v_elegido !== 'undefined') {
                    this.datosTransporte.GGVEHICULO_COD = v_elegido.COD;
                    this.datosTransporte.GGVEHICULO_DESCRIP = v_elegido.DESCRIP;
                    this.actualizaVista();
                }
            }
        }

        return DialogoDatosTransporte;
    });

