﻿// @ts-nocheck
// lodash
define('dialogo-lotes-ubicaciones',
    [
        'componentes',
    ],
    function (
        componentes,
    ) {
        let notificacion = componentes.notificacion;

        class DialogoLotesUbicaciones {
            constructor(p_modal, p_linea, p_disponible) {
                this.modal = p_modal;

                this.gaart_cod = p_linea.GAART_COD;
                this.gaart_descrip = p_linea.GAART_DESCRIP;
                this.pendiente = p_linea.C_PDTE;

                this.pendienteOPreparado = 'pendiente';

                this.linea = _.cloneDeep(p_linea);
                this.lineaOriginal = p_linea;

                this.disponible = _.cloneDeep(p_disponible);
                this.disponibleOriginal = p_disponible;

                this.gaubic_cod = this.lote = this.cantidad = null;

                this._inputGaart_cod = p_modal.campo('gaart_cod').crea();
                this._inputGaart_descrip = p_modal.campo('gaart_descrip').crea();
                this._botonPendienteOPreparado = p_modal.boton('pendiente-o-preparado')
                    .clic(this.muestraPendienteOPreparado.bind(this))
                    .crea();
                this._inputPendiente = p_modal.campo('pendiente').crea();

                let v_tieneControlLotes = p_linea.GAART_LOTES_SN === 'S';
                let v_tieneControlUbic = p_linea.GAART_UBIC_SN === 'S';

                let v_columnas = [];
                if (v_tieneControlUbic)
                    v_columnas = v_columnas.concat([{ propiedad: 'GAUBIC_COD', titulo: 'Ubicación', ancho: 'auto' }]);
                if (v_tieneControlLotes)
                    v_columnas = v_columnas.concat([{ propiedad: 'LOTE', titulo: 'Lote', ancho: 'auto' }]);
                v_columnas = v_columnas.concat([{ propiedad: 'C_UNI1', titulo: 'Disp.', ancho: '20%', numerico: true }]);

                this._tabla = p_modal.tabla('disponible')
                    .columnas(v_columnas)
                    .seleccionado(p_seleccion => {
                        this.gaubic_cod = p_seleccion && p_seleccion.GAUBIC_COD || null;
                        this.lote = p_seleccion && p_seleccion.LOTE || null;
                        this.cantidad = Math.min(p_seleccion && p_seleccion.C_UNI1, this.pendiente) || null;
                        this.actualizaVista();
                        this._inputCantidad.foco();
                    })
                    .formatoFila((p_formato, p_disponible) => {
                        let v_indice = this.disponible.findIndex(p => _.isEqual(p, p_disponible));
                        if (v_indice === -1)
                            throw new Error('????');
                        if (!_.isEqual(this.disponible[v_indice], this.disponibleOriginal[v_indice]))
                            p_formato.fondo('verde');
                    })
                    .crea();

                this.bloqueEdicion = p_modal.bloque('bloque-edicion')
                    .crea();

                this._inputGaubic_cod = p_modal.campo('gaubic_cod')
                    .cambio(p_valor => {
                        let v_lineas = this._tabla.buscaYSelecciona({ GAUBIC_COD: p_valor, LOTE: this._inputLote.valor() }, { seleccionaPrimero: true });
                        if (!Array.isArray(v_lineas) || v_lineas.length === 0) {
                            notificacion().error().muestra('No es una ubicación válida');
                            this._inputGaubic_cod.foco();
                        }
                    })
                    .crea();
                this._inputGaubic_cod.visible(v_tieneControlUbic);


                this._inputLote = p_modal.campo('lote')
                    .cambio(p_valor => {
                        let v_lineas = this._tabla.buscaYSelecciona({ GAUBIC_COD: this._inputGaubic_cod.valor(), LOTE: p_valor }, { seleccionaPrimero: true });
                        if (!Array.isArray(v_lineas) || v_lineas.length === 0) {
                            notificacion().error().muestra('No es un lote válido');
                            this._inputGaubic_cod.foco();
                        }
                    })
                    .crea();
                this._inputLote.visible(v_tieneControlLotes);


                this._inputCantidad = p_modal.campo('cantidad')
                    .cambio(
                        p_valor => {
                            let v_gaubic_cod = this._inputGaubic_cod.valor();
                            let v_lote = this._inputLote.valor();
                            this.insertaOActualizaLinea(v_gaubic_cod, v_lote, p_valor);
                        }
                    )
                    .valida(p_evento => {
                        let v_gaubic_cod = this._inputGaubic_cod.valor();
                        let v_lote = this._inputLote.valor();
                        this.insertaOActualizaLinea(v_gaubic_cod, v_lote, p_evento.nuevoValor);
                    })
                    .crea();

                this._botonAceptar = p_modal.boton('aceptar')
                    .clic(this.acepta.bind(this))
                    .crea();
            }

            puedeAceptar() {
                let v_puedeAceptar = !_.isEqual(this.linea, this.lineaOriginal) || (this.gaubic_cod !== null && this.lote !== null && this.cantidad !== null);
                return v_puedeAceptar;
            }

            insertaOActualizaLinea(p_gaubic_cod, p_lote, p_cantidad) {
                var v_disponible = this.disponible.find(p_disponible => p_disponible.GAUBIC_COD === p_gaubic_cod && p_disponible.LOTE === p_lote);
                if (typeof v_disponible !== 'undefined')
                    v_disponible.C_UNI1 -= p_cantidad;

                this.pendiente -= p_cantidad;
                this.linea.C_PREP += p_cantidad;
                let v_loteUbicPreparado = this.linea.Detalles.find(p_preparado => p_preparado.GAUBIC_COD === p_gaubic_cod && p_preparado.LOTE === p_lote);
                if (typeof v_loteUbicPreparado !== 'undefined')
                    v_loteUbicPreparado.C_UNI1 += p_cantidad;
                else
                    this.linea.Detalles.push({
                        GAUBIC_COD: p_gaubic_cod,
                        LOTE: p_lote,
                        C_UNI1: p_cantidad
                    });
                //if (v_loteUbicPreparado.length === 0) 
                //    error().notificacion().muestra('Debe indicar una ubicación y lote válidos');
                //else

                this.gaubic_cod = this.lote = this.cantidad = null;

                this.actualizaVista();
            }

            muestraPendienteOPreparado() {
                if (this.pendienteOPreparado === 'pendiente')
                    this.pendienteOPreparado = 'preparado';
                else
                    this.pendienteOPreparado = 'pendiente';
                this.actualizaVista();
            }

            actualizaVista() {
                this._inputGaart_cod.valor(this.gaart_cod);
                this._inputGaart_descrip.valor(this.gaart_descrip);

                if (this.pendienteOPreparado === 'pendiente') {
                    this._botonPendienteOPreparado.etiqueta('Pdte.');
                    this._inputPendiente.valor(this.pendiente);
                }
                else {
                    this._botonPendienteOPreparado.etiqueta('Prep.');
                    this._inputPendiente.valor(this.linea.C_PREP);
                }

                this._tabla.datos(this.disponible);
                if (this._tablaPreparado)
                    this._tablaPreparado.datos(this.linea.Detalles);

                this.bloqueEdicion.clase('ubicacion', this.linea.GAART_UBIC_SN === 'S');
                this.bloqueEdicion.clase('lote', this.linea.GAART_LOTES_SN === 'S');

                this._inputGaubic_cod.valor(this.gaubic_cod);
                this._inputLote.valor(this.lote);
                this._inputCantidad.valor(this.cantidad);

                this._botonAceptar.habilitado(this.puedeAceptar());
            }

            actualizaModelo() {
                this.gaubic_cod = this._inputGaubic_cod.valor();
                this.lote = this._inputLote.valor();
                this.cantidad = this._inputCantidad.valor();
            }

            acepta() {
                this.actualizaModelo();

                if (typeof this.gaubic_cod !== 'undefined' && typeof this.lote !== 'undefined' && typeof this.cantidad !== 'undefined')
                    this.insertaOActualizaLinea(this.gaubic_cod, this.lote, this.cantidad);

                this.modal.cierra({
                    disponible: this.disponible,
                    preparado: {
                        C_UNI1: this.linea.C_PREP,
                        Detalles: this.linea.Detalles
                    }
                });
            }
        }

        return DialogoLotesUbicaciones;
    });