﻿// @ts-nocheck
// lodash
define('dialogo-resumen-bultos',
    [
        'api',
        'componentes',
        'dialogo-consulta-bultos'
    ],
    function (
        api,
        componentes,
        DialogoConsultaBultos
    ) {
        let
            componente = componentes.componente,
            boton = componentes.boton,
            lov = componentes.lov,
            error = componentes.error,
            dialogo = componentes.dialogo;

        class DialogoResumenBultos {
            constructor(p_modal, p_resumenBultos, p_otros) {
                this.modal = p_modal;

                this.resumenBultos = Object.assign({ N_COPIAS: null }, _.cloneDeep(p_resumenBultos));

                let {
                    mostrarAlbaranValorado: v_mostrarAlbaranValorado,
                    mostrarCopias: v_mostrarCopias,
                    detalleBultos: v_detalleBultos
                } = p_otros || {};

                this.bultos = v_detalleBultos;

                this._botonConfirmar = p_modal.componente('confirmar')
                    .clic(this.confirma.bind(this))
                    .tecla('Enter')
                    .crea();
                this._botonImprimir = p_modal.componente('imprimir').clic(this.imprime.bind(this)).crea();

                this._inputGgmenvio_cod = p_modal.componente('ggmenvio_cod')
                    .cambio(this.validaGgmenvio_cod.bind(this))
                    .lov(this.pideGgmenvio_cod.bind(this))
                    .crea();
                this._inputGgmenvio_descrip = p_modal.componente('ggmenvio_descrip')
                    .crea();
                this._botonBusquedaGgmenvio_cod = p_modal.boton('busqueda-ggmenvio_cod')
                    .clic(this.pideGgmenvio_cod.bind(this))
                    .crea();

                this._inputGgvehiculo_cod = p_modal.componente('ggvehiculo_cod')
                    .cambio(this.validaGgvehiculo_cod.bind(this))
                    .lov(this.pideGgvehiculo_cod.bind(this))
                    .crea();
                this._inputGgvehiculo_descrip = p_modal.componente('ggvehiculo_descrip')
                    .crea();
                this._botonBusquedaGgvehiculo_cod = p_modal.boton('busqueda-ggvehiculo_cod')
                    .clic(this.pideGgvehiculo_cod.bind(this))
                    .crea();

                this._botonConsultaBultos = p_modal.componente('consulta-bultos')
                    .clic(this.consultaBultos.bind(this))
                    .crea();

                this.etiquetaBultos = p_modal.componente('etiqueta-bultos')
                    .crea();

                this._inputBultos = p_modal.componente('bultos').crea();
                this._inputPeso = p_modal.componente('peso').crea();
                this._inputEtiquetas = p_modal.componente('etiquetas').crea();
                this._desplegablePlantillas = p_modal.componente('plantilla')
                    .valores(p_resumenBultos.Opciones)
                    .crea();

                this._inputCopias = p_modal.componente('copias').crea();
                this._inputCopias.visible(v_mostrarCopias);

                this._inputAlbaranValorado = p_modal.componente('albaran-valorado').crea();
                this._inputAlbaranValorado.visible(v_mostrarAlbaranValorado);
            }

            actualizaVista() {
                this._inputGgmenvio_cod.valor(this.resumenBultos.GGMENVIO_COD);
                this._inputGgmenvio_descrip.valor(this.resumenBultos.GGMENVIO_DES);
                this._inputGgvehiculo_cod.valor(this.resumenBultos.GGVEHICULO_COD);
                this._inputGgvehiculo_descrip.valor(this.resumenBultos.GGVEHICULO_DES);
                this._inputBultos.valor(this.resumenBultos.N_BULTOS);
                this._inputPeso.valor(this.resumenBultos.PESO);
                this._inputEtiquetas.valor(this.resumenBultos.N_ETIQUETAS);
                this._desplegablePlantillas.valor(this.resumenBultos.OpcionPredeterminada);
                this._inputCopias.valor(this.resumenBultos.N_COPIAS);
                this._inputAlbaranValorado.valor(this.resumenBultos.AlbaranValorado);

                this._botonConsultaBultos.visible(this.bultos);
                this.etiquetaBultos.visible(!this.bultos);
            }

            actualizaModelo() {
                this.resumenBultos.GGVEHICULO_COD = this._inputGgvehiculo_cod.valor().toUpperCase();
                this.resumenBultos.N_BULTOS = this._inputBultos.valor();
                this.resumenBultos.PESO = this._inputPeso.valor();
                this.resumenBultos.N_ETIQUETAS = this._inputEtiquetas.valor();
                this.resumenBultos.OpcionPredeterminada = this._desplegablePlantillas.valor();
                this.resumenBultos.N_COPIAS = this._inputCopias.valor();
                this.resumenBultos.AlbaranValorado = this._inputAlbaranValorado.valor();
            };

            confirma() {
                this.actualizaModelo();
                this.modal.cierra([this.resumenBultos, { confirma: true, imprimeEtiquetas: false }]);
            }

            imprime() {
                this.actualizaModelo();
                this.modal.cierra([this.resumenBultos, { confirma: true, imprimeEtiquetas: true }]);
            }

            async validaGgmenvio_cod() {
                this.actualizaModelo();

                let v_modoEnvio;
                try {
                    [v_modoEnvio] = await api('/Expedicion/ModoEnvio')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            ggmenvio_cod: this.resumenBultos.GGMENVIO_COD
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await error(v_mensaje).muestra();
                    return;
                }

                this.resumenBulto.GGMENVIO_COD = v_modoEnvio.COD;
                this.resumenBulto.GGMENVIO_DES = v_modoEnvio.DESCRIP;
                this.resumenBulto.GGVEHICULO_COD = this.resumenBulto.GGVEHICULO_DES = null;

                this.actualizaVista();
            }

            async pideGgmenvio_cod() {
                let v_modosEnvio;
                try {
                    [v_modosEnvio] = await api('/Expedicion/ModosEnvio')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD")
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await error(v_mensaje).muestra();
                    return;
                }

                const v_elegido = await lov()
                    .titulo('Lista de transportistas')
                    .columnas([
                        { propiedad: 'COD', titulo: 'Código', ancho: '6em' },
                        { propiedad: 'DESCRIP', titulo: 'Nombre' }
                    ])
                    .elige(v_modosEnvio);

                if (typeof v_elegido !== 'undefined') {
                    this.resumenBultos.GGMENVIO_COD = v_elegido.COD;
                    this.resumenBultos.GGMENVIO_DES = v_elegido.DESCRIP;
                    this.resumenBultos.GGVEHICULO_COD = this.resumenBultos.GGVEHICULO_DES = null;
                    this.actualizaVista();
                }
            }

            async validaGgvehiculo_cod() {
                this.actualizaModelo();

                var v_vehiculo;
                try {
                    [v_vehiculo] = await api('/Expedicion/Vehiculo')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            ggmenvio_cod: this.resumenBultos.GGMENVIO_COD,
                            cod: this.resumenBultos.GGVEHICULO_COD
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await error(v_mensaje).muestra();
                    return;
                }

                this.resumenBulto.GGVEHICULO_COD = v_vehiculo.COD;
                this.resumenBulto.GGVEHICULO_DES = v_vehiculo.DESCRIP;

                this.actualizaVista();
            }

            async pideGgvehiculo_cod() {
                let v_vehiculos;
                try {

                    [v_vehiculos] = await api('/Expedicion/Vehiculos')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            ggmenvio_cod: this.resumenBultos.GGMENVIO_COD
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await error(v_mensaje).muestra();
                    return;
                }

                let v_elegido = await lov()
                    .titulo('Lista de transportistas')
                    .columnas([
                        { propiedad: 'COD', titulo: 'Código', ancho: '6em' },
                        { propiedad: 'DESCRIP', titulo: 'Nombre' }
                    ])
                    .elige(v_vehiculos);

                if (typeof v_elegido !== 'undefined') {
                    this.resumenBultos.GGVEHICULO_COD = v_elegido.COD;
                    this.resumenBultos.GGVEHICULO_DES = v_elegido.DESCRIP;
                    this.actualizaVista();
                }
            }

            async consultaBultos() {
                await dialogo()
                    .titulo('Bultos')
                    .plantilla('dialogo-consulta-bultos')
                    .construye(p_modal => new DialogoConsultaBultos(p_modal, this.bultos))
                    .abre();
            }
        }

        return DialogoResumenBultos;
    });

