﻿// @ts-nocheck
define('dialogo-seleccion-albaran',
    [
        'api',
        'componentes',
        'foco'
    ],
    function (
        api,
        componentes,
        foco
    ) {

        let
            boton = componentes.boton,
            campo = componentes.campo,
            tabla = componentes.tabla,
            error = componentes.error;

        class DialogoSeleccionAlbaran {
            constructor(p_modal, p_albaranes) {
                this.modal = p_modal;

                this.albaranes = p_albaranes.map(p_albaran => Object.assign(p_albaran, { F_ALBARAN: moment(p_albaran.F_ALBARAN).format("DD/MM/YY") })); // en lugar de DD/MM/YY => Globalize($("html").attr("lang")).culture().calendar.patterns.d.toUpperCase()
                this.lecturaAlbaran = null;

                this.ggalbtip_cod = this.n_albaran = this.f_albaran = null;

                this._inputAlbaran = p_modal.campo('lecturaAlbaran')
                    .cambio(this.validaAlbaran.bind(this))
                    .crea();

                this._tabla = p_modal.tabla('albaranes')
                    .columnas([
                        { propiedad: 'GGALBTIP_COD', titulo: 'Tipo', ancho: 'auto' },
                        { propiedad: 'N_ALBARAN', titulo: 'No.', ancho: 'auto', numerico: true },
                        { propiedad: 'F_ALBARAN', titulo: 'Fecha', ancho: 'auto', alineacion: 'centro' },
                        { propiedad: 'GGESTADO_COD', titulo: 'Estado', ancho: 'auto', alineacion: 'centro' },
                    ])
                    .seleccionado(p_seleccion => {
                        this.ggalbtip_cod = p_seleccion && p_seleccion.GGALBTIP_COD || null;
                        this.n_albaran = p_seleccion && p_seleccion.N_ALBARAN || null;
                        this.f_albaran = p_seleccion && p_seleccion.F_ALBARAN || null;
                        this.actualizaVista();
                    })
                    .crea();

                this._botonNuevo = p_modal.boton('nuevo')
                    .clic(this.nuevo.bind(this))
                    //.tecla('F7')
                    .crea();

                this._botonSeleccionar = p_modal.boton('seleccionar')
                    .clic(this.seleccionar.bind(this))
                    .tecla('Enter')
                    .crea();
            }

            actualizaVista() {
                this._inputAlbaran.valor(this.lecturaAlbaran);
                this._botonSeleccionar.habilitado(this.ggalbtip_cod && this.n_albaran && this.f_albaran);
                this._tabla.datos(this.albaranes);
            }

            actualizaModelo() {
                this.lecturaAlbaran = this._inputAlbaran.valor();
            }

            async validaAlbaran(p_valor) {
                if (typeof p_valor !== 'string' || p_valor.length === 0)
                    return;

                var v_albaran;
                try {
                    [v_albaran] = await api('/Expedicion/NumeroAlbaran')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            lectura_albaran: p_valor.toUpperCase()
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await error(v_mensaje).muestra();
                    return;
                }
            }

            nuevo() {
                this.ggalbtip_cod = this.n_albaran = this.f_albaran = null;
                this.modal.cierra('nuevo');
            }

            seleccionar() {
                this.actualizaModelo();
                this.modal.cierra({
                    GGALBTIP_COD: this.ggalbtip_cod,
                    N_ALBARAN: this.n_albaran,
                    F_ALBARAN: this.f_albaran
                });
            }
        }

        return DialogoSeleccionAlbaran;
    });