﻿// @ts-nocheck
define('dialogo-seleccion-bulto',
    [
        'api',
        'componentes',
        'foco',
        'dialogo-seleccion-tipo-bulto',
        'dialogo-detalle-bulto'
    ],
    function (
        api,
        componentes,
        foco, 
        DialogoSeleccionTipoBulto,
        DialogoDetalleBulto
    ) {
        let
            boton = componentes.boton,
            campo = componentes.campo,
            tabla = componentes.tabla,
            error = componentes.error,
            lov = componentes.lov,
            dialogo = componentes.dialogo;

        class DialogoSeleccionBulto {
            constructor(p_modal, p_bultos, p_cantidad) {
                this.modal = p_modal;

                this.bultos = p_bultos.map(p_bulto => ({
                    N_BULTO: p_bulto.N_BULTO,
                    GVPALLETIP_COD: p_bulto.GVPALLETIP.COD,
                    GVPALLETIP_DESCRIP: p_bulto.GVPALLETIP.DESCRIP,
                    CANTIDAD: p_bulto.articulos.map(p_articulo => p_articulo.CANTIDAD).reduce((p_total, p_cantidad) => p_total + p_cantidad, 0),
                    articulos: p_bulto.articulos
                }));

                this.cantidad = p_cantidad;

                this.tabla = p_modal.tabla('bultos')
                    .seleccionado(this.seleccionaBulto.bind(this))
                    .columnas([
                        { propiedad: 'N_BULTO', titulo: 'Nº', ancho: '2em', numerico: true },
                        { propiedad: 'GVPALLETIP_COD', titulo: 'Bulto', ancho: '3em' },
                        { propiedad: 'GVPALLETIP_DESCRIP', titulo: 'Descripcion', ancho: 'auto' },
                        { propiedad: 'CANTIDAD', titulo: 'Cantidad', ancho: '6em', numerico: true, },
                    ])
                    .crea();

                this.botonSeleccionar = p_modal.boton('seleccionar')
                    .clic(this.seleccionar.bind(this))
                    .tecla('Enter')
                    .crea();

                this.botonNinguno = p_modal.boton('ninguno')
                    .clic(this.ninguno.bind(this))
                    .crea();

                this.botonNuevo = p_modal.boton('nuevo')
                    .clic(this.nuevo.bind(this))
                    .crea();

                this.botonDetalles = p_modal.boton('detalles')
                    .clic(this.detalles.bind(this))
                    .crea();
            }

            seleccionaBulto(p_bulto) {
                this.seleccionado = p_bulto;
                this.actualizaVista();
            }

            actualizaVista() {
                this.tabla.datos(this.bultos);
                this.botonSeleccionar.habilitado(typeof this.seleccionado !== 'undefined');
                this.botonDetalles.habilitado(typeof this.seleccionado !== 'undefined');
            }

            seleccionar() {
                let v_seleccionado = this.seleccionado;

                this.modal.cierra({ opcion: 'actualiza', bulto: v_seleccionado });
            }

            ninguno() { 
                this.modal.cierra({ opcion: 'ninguno' });
            }

            async nuevo() {
                let v_tiposBulto;
                try {
                    [v_tiposBulto] = await api('/Expedicion/TiposPale')
                        .parametros({
                            EMPGRUPO_COD: sessionStorage.getItem("EMPRGRUP_COD"),
                            EMP_COD: sessionStorage.getItem("EMPR_COD")
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await error(v_mensaje).muestra();
                    return;
                }

                //let v_elegido = await lov()
                //    .titulo('Nuevo bulto')
                //    .columnas([
                //        { propiedad: 'COD', titulo: 'Código', ancho: '6em' },
                //        { propiedad: 'DESCRIP', titulo: 'Nombre' }
                //    ])
                //    .elige(v_tiposBulto);


                const v_resultadoSeleccionTipoBulto = await dialogo()
                    .titulo('Nuevo bulto')
                    .plantilla('dialogo-seleccion-tipo-bulto')
                    .construye(p_modal => new DialogoSeleccionTipoBulto(p_modal, v_tiposBulto, this.cantidad))
                    .abre();

                if (typeof v_resultadoSeleccionTipoBulto === 'undefined')
                    return;

                this.modal.cierra(Object.assign({ opcion: 'nuevo' }, v_resultadoSeleccionTipoBulto));
            }

            async detalles() {
                let v_seleccionado = this.seleccionado;

                await dialogo()
                    .titulo('Detalle de bulto')
                    .plantilla('dialogo-detalle-bulto')
                    .construye(p_modal => new DialogoDetalleBulto(p_modal, v_seleccionado))
                    .abre();
            }
        }

        return DialogoSeleccionBulto;
    });