﻿// @ts-nocheck
// lodash
define('pantalla-expedicion',
    [
        'api',
        'componentes',
        'foco',
        'dialogo-seleccion-albaran',
        'dialogo-lotes-ubicaciones',
        'dialogo-datos-transporte',
        'dialogo-resumen-bultos',
        'dialogo-seleccion-bulto',
        'dialogo-consulta-bultos',
        'dialogo-prueba-tablas',
    ],

    function (
        api,
        componentes,
        foco,
        DialogoSeleccionAlbaran,
        DialogoLotesUbicaciones,
        DialogoDatosTransporte,
        DialogoResumenBultos,
        DialogoSeleccionBulto,
        DialogoConsultaBultos,
        DialogoPruebaTablas
    ) {

        let
            boton = componentes.boton,
            campo = componentes.campo,
            dialogo = componentes.dialogo,
            mensaje = componentes.mensaje,
            notificacion = componentes.notificacion,
            pregunta = componentes.pregunta,
            lov = componentes.lov,
            tabla = componentes.tabla,
            bloque = componentes.bloque,
            cortina = componentes.cortina;

        class PantallaExpedicion {
            constructor() {
                //this.lecturaPedido = null;
                //this.lecturaArticulo = null;
                //this.lecturaCantidad = null;
                //this.pedido = null;
                //this.pedidoPreparado = null;
                //this.lineaSeleccionada = null;

                this.inicializa();
            }

            inicializaComponentes() {
                foco.enPrimero(document);

                return {
                    botonNuevo:
                        boton('btnNuevoPedido')
                            .clic(() => {
                                this.nuevoPedido();
                                this.actualizaVista();
                            }),

                    botonConfirmar:
                        boton('confirmar')
                            .clic(async () => {
                                this.actualizaModelo();
                                await this.confirmaPedido({ dejarPendiente: false });
                                this.actualizaVista();
                            })
                            .tecla('Enter'),

                    botonPendiente:
                        boton('btnPdte')
                            .clic(async () => {
                                this.actualizaModelo();
                                await this.confirmaPedido({ dejarPendiente: true });
                                this.actualizaVista();
                            }),

                    tablaLineas:
                        tabla('tablaLineas')
                            .columnas([
                                //{ propiedad: 'N_LIN', titulo: 'Línea', ancho: 'auto' },
                                //{ propiedad: 'GAART_UBIC_SN', titulo: 'No.', ancho: 'auto' },
                                //{ propiedad: 'GAART_LOTES_SN', titulo: 'Fecha', ancho: 'auto' },
                                { propiedad: 'GAART_COD', titulo: 'Artículo', ancho: '6em' },
                                { propiedad: 'GAART_DESCRIP', titulo: 'Descripción', ancho: 'auto' },
                                { propiedad: 'C_PDTE', titulo: 'Pdte.', ancho: '4em', numerico: true },
                                //{ opcion: 'elimina', ancho: '5%' }
                            ])
                            //.opciones({
                            //    elimina: { accion: this.hazAlgo.bind(this), boton: 'elimina' }
                            //})
                            .seleccionado(p_seleccion => this.seleccionaLinea(p_seleccion))
                            .formatoFila((p_formato, p_linea) => {
                                if (p_linea.C_PREP !== 0)
                                    p_formato.fondo('verde');
                            }),

                    campoLecturaPedido:
                        campo('txtPedido')
                            .cambio(async () => {
                                this.actualizaModelo();
                                await this.cargaPedido();
                                this.actualizaVista();
                            }),

                    campoGGCLIPR_COD:
                        campo('txtCliente'),

                    campoGGCLIPR_NOMBRE:
                        campo('txtNomCliente'),

                    campoReferencia:
                        campo('campRefer')
                            .cambio(async () => {
                                this.actualizaModelo();
                                this.obtenerLinea();
                                this.actualizaVista();
                            }),

                    bloqueReferencia:
                        bloque('ref'),

                    campoC_PREP:
                        campo('campCant')
                            .cambio(this.actualizaPreparadoArticuloSinLotesNiUbicaciones.bind(this))
                };
            }

            //hazAlgo(p) {
            //    const v_indice = this.pedidoPreparado.Lineas.indexOf(p);
            //    this.pedidoPreparado.Lineas.splice(v_indice, 1);
            //    this.tablaLineas.refresca();
            //    this.actualizaVista();
            //}

            generaCortina() {
                let v_cortina = cortina();

                if (this.pedido === null || this.pedidoPreparado === null)
                    return v_cortina;

                // opcion hoy por hoy puramente informativa
                // pendiente de implementar la opcion de cambiar la fecha del albarán a generar
                v_cortina = v_cortina
                    .opcionFecha('fecha-albaran', p_opcion =>
                        p_opcion
                            .titulo('Fecha de albarán')
                            .icono('fecha')
                            .cambio(p_valor => this.pedidoPreparado.F_ALBARAN = p_valor)
                    );

                if (Array.isArray(this.pedido.Albaranes) && this.pedido.Albaranes.length > 0) {
                    v_cortina = v_cortina
                        .opcion('cambiar-albaran', p_opcion =>
                            p_opcion
                                .titulo('Cambiar albarán')
                                .icono('observaciones')
                                .iconoValor('buscador')
                                .clic(async p_opcion => {
                                    let v_resultadoSeleccionAlbaran = await this.pideAlbaran(this.pedido.Albaranes);
                                    if (typeof v_resultadoSeleccionAlbaran === 'undefined')
                                        return;
                                    this.seleccionaAlbaran(v_resultadoSeleccionAlbaran);
                                    this.actualizaVista();
                                })
                        );
                }

                //v_cortina = v_cortina
                //    .opcion('su-referencia', p_opcion =>
                //        p_opcion
                //            .titulo('Referencia de albarán')
                //            .icono('observaciones')
                //            .valor(v_su_ref_alb)
                //    );


                if (typeof this.pedido.OBSER === 'string' && this.pedido.OBSER.length > 0) {
                    v_cortina = v_cortina
                        .opcion('observacion', p_opcion =>
                            p_opcion
                                .titulo('Observaciones')
                                .icono('observaciones')
                                .valor('Mostrar')
                                .iconoValor('abrir')
                                .clic(async () => await mensaje(this.pedido.OBSER).muestra())
                        );
                }

                v_cortina = v_cortina
                    .opcion('bultos', p_opcion =>
                        p_opcion
                            .titulo('Bultos')
                            .icono('observaciones')
                            //.valor('Mostrar')
                            .iconoValor('abrir')
                            .clic(async () => {
                                await dialogo()
                                    .titulo('Bultos')
                                    .plantilla('dialogo-consulta-bultos')
                                    .construye(p_modal => new DialogoConsultaBultos(p_modal, this.pedidoPreparado.Bultos))
                                    .abre();
                            })
                    );

                return v_cortina;
            }

            actualizaVista() {
                if (typeof this.botonNuevo === 'undefined') {
                    let v_componentes = this.inicializaComponentes();
                    Object.getOwnPropertyNames(v_componentes).forEach(p_nombre => this[p_nombre] = v_componentes[p_nombre].crea());
                }

                this.campoLecturaPedido.valor(this.lecturaPedido);
                this.campoGGCLIPR_COD.valor(this.pedido === null ? null : this.pedido.GGCLIPR_COD);
                this.campoGGCLIPR_NOMBRE.valor(this.pedido === null ? null : this.pedido.GGCLIPR_NOMBRE);

                this.tablaLineas.datos(this.pedidoPreparado && this.pedidoPreparado.Lineas);
                if (this.pedidoPreparado && Array.isArray(this.pedidoPreparado.Lineas)) {
                    let v_indices = this.pedidoPreparado.Lineas
                        .map((p_linea, p_indice) => ([p_linea, p_indice]))
                        .filter(([p_linea, p_indice]) => p_linea.C_PREP !== 0)
                        .map(([p_linea, p_indice]) => p_indice);
                }
                this.tablaLineas.buscaYSelecciona(this.lineaSeleccionada, { notificar: false });

                this.campoReferencia.valor(this.lecturaArticulo);
                this.campoC_PREP.valor(this.lecturaCantidad);

                let v_sePuedeConfirmarPedido = this.sePuedeConfirmarPedido();
                this.botonConfirmar.habilitado(v_sePuedeConfirmarPedido);
                this.botonPendiente.habilitado(v_sePuedeConfirmarPedido);
                this.campoLecturaPedido.habilitado(!this.pedidoPreparado);
                this.campoReferencia.habilitado(this.pedidoPreparado);

                this.bloqueReferencia.clase('solo-referencia', !this.lineaSeleccionada || (this.lineaSeleccionada.GAART_LOTES_SN === 'S' || this.lineaSeleccionada.GAART_UBIC_SN === 'S'))

                this.campoReferencia.valor(this.lecturaArticulo);

                if (this.pedidoPreparado) {
                    let v_opcionFechaAlbaran = this.cortina.opciones['fecha-albaran'];
                    if (v_opcionFechaAlbaran)
                        v_opcionFechaAlbaran.valor(
                            this.pedidoPreparado.F_ALBARAN ? this.pedidoPreparado.F_ALBARAN : moment(new Date()).format("DD/MM/YYYY")
                        );

                    let v_opcionCambiarAlbaran = this.cortina.opciones['cambiar-albaran'];
                    if (v_opcionCambiarAlbaran)
                        v_opcionCambiarAlbaran.valor(
                            this.pedidoPreparado.GGALBTIP_COD && this.pedidoPreparado.N_ALBARAN
                                ? `${this.pedidoPreparado.GGALBTIP_COD}-${this.pedidoPreparado.N_ALBARAN}`
                                : 'Nuevo'
                        );

                    let v_opcionBultos = this.cortina.opciones['bultos'];
                    if (v_opcionBultos) {
                        let v_texto;
                        switch (this.pedidoPreparado.Bultos.length) {
                            case 0:
                                v_texto = 'Sin bultos';
                                break;
                            case 1:
                                v_texto = '1 bulto';
                                break;
                            default:
                                v_texto = `${this.pedidoPreparado.Bultos.length} bultos`;
                                break;
                        }
                        v_opcionBultos.valor(v_texto);
                    }
                }
            }

            async nuevoPedido() {
                this.inicializa();
                this.campoLecturaPedido.foco();
                return;
            }

            actualizaModelo() {
                this.lecturaPedido = this.campoLecturaPedido.valor().toUpperCase();
                this.lecturaArticulo = this.campoReferencia.valor().toUpperCase();
                this.lecturaCantidad = this.campoC_PREP.valor();
            }

            inicializa() {
                this.lecturaPedido = null;
                this.lecturaArticulo = null;
                this.lecturaCantidad = null;
                this.pedido = null;
                this.pedidoPreparado = null;
                this.lineaSeleccionada = null;

                // reinicio la cortina
                let v_cortina = this.generaCortina();
                this.cortina = v_cortina.activa();
            }

            async cargaPedido() {
                let v_pedido;
                try {
                    [v_pedido] = await api('/Expedicion/Pedido')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            lecturaPedido: this.lecturaPedido
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje().error().muestra(v_mensaje);
                    return;
                }

                // si hay muestro la observación
                if (typeof v_pedido.OBSER === 'string' && v_pedido.OBSER.length > 0) {
                    let v_respuesta = await mensaje(v_pedido.OBSER).muestra();
                    if (typeof v_respuesta === 'undefined') {
                        this.inicializa();
                        this.campoLecturaPedido.foco();
                        return;
                    }
                }

                // solicito datos de transporte
                let v_resultadoDatosTransporte;
                if (v_pedido.DatosEnvioInicial) {
                    let v_datosTransporte = {
                        GGMENVIO_COD: v_pedido.GGMENVIO_COD,
                        GGMENVIO_DESCRIP: v_pedido.GGMENVIO_DESCRIP,
                        GGVEHICULO_COD: v_pedido.GGVEHICULO_COD,
                        GGVEHICULO_DESCRIP: v_pedido.GGVEHICULO_DESCRIP
                    };

                    v_resultadoDatosTransporte = await dialogo()
                        .titulo('Datos de envío')
                        .plantilla('dialogo-datos-transporte')
                        .construye(p_modal => new DialogoDatosTransporte(p_modal, v_datosTransporte))
                        .abre();
                    if (typeof v_resultadoDatosTransporte === 'undefined') {
                        this.inicializa();
                        this.campoLecturaPedido.foco();
                        return;
                    }
                }
                else {
                    v_resultadoDatosTransporte = {
                        GGMENVIO_COD: v_pedido.GGMENVIO_COD,
                        GGMENVIO_DESCRIP: v_pedido.GGMENVIO_DESCRIP,
                        GGVEHICULO_COD: v_pedido.GGVEHICULO_COD,
                        GGVEHICULO_DESCRIP: v_pedido.GGVEHICULO_DESCRIP,
                    };
                }

                // si hay albaranes para el pedido, solicito uno. 
                let v_resultadoSeleccionAlbaran = await this.pideAlbaran(v_pedido.Albaranes);
                if (typeof v_resultadoSeleccionAlbaran === 'undefined') {
                    this.inicializa();
                    this.campoLecturaPedido.foco();
                    return;
                }

                // preparo el modelo
                this.pedido = v_pedido;

                this.disponible = this.pedido.Lineas
                    .reduce((p_diccionario, p_linea) => {
                        p_diccionario[p_linea.GAART_COD] = p_linea.Existencias;
                        return p_diccionario;
                    }, {});

                this.pedidoPreparado = {
                    EMPGRUPO_COD: sessionStorage.getItem("EMPRGRUP_COD"),
                    EMP_COD: sessionStorage.getItem("EMPR_COD"),
                    GGPEDTIP_COD: v_pedido.GGPEDTIP_COD,
                    N_PEDIDO: v_pedido.N_PEDIDO,

                    GGMENVIO_COD: v_resultadoDatosTransporte.GGMENVIO_COD,
                    GGMENVIO_DES: v_resultadoDatosTransporte.GGMENVIO_DESCRIP,
                    GGVEHICULO_COD: v_resultadoDatosTransporte.GGVEHICULO_COD,
                    GGVEHICULO_DES: v_resultadoDatosTransporte.GGVEHICULO_DESCRIP,

                    Lineas: v_pedido.Lineas.map(p_linea => ({
                        N_LIN: p_linea.N_LIN,
                        GAART_COD: p_linea.GAART_COD,
                        GAART_DESCRIP: p_linea.GAART_DESCRIP,
                        C_PREP: 0,
                        C_PDTE: p_linea.C_PDTE,
                        C_PDTE2: p_linea.C_PDTE2,
                        C_UNI1: 0,
                        C_UNI2: null,
                        GAART_LOTES_SN: p_linea.GAART_LOTES_SN,
                        GAART_UBIC_SN: p_linea.GAART_UBIC_SN,
                        Detalles: []
                    })),

                    N_ETIQUETAS: 0, // int 
                    PESO: 0, // decimal;
                    N_BULTOS: 0, // int 
                    Bultos: [], // Bulto[]  

                    N_COPIAS: null, // int?
                    DejarPendiente: null, // bool?
                    ImprimirValorado: null, // bool? 
                    AlbaranValorado: v_pedido.AlbaranValorado,
                    OPCION_ETIQUETAS: null, // string 
                    ImprimirEtiquetas: false, // bool
                };

                this.seleccionaAlbaran(v_resultadoSeleccionAlbaran);

                const v_cortina = this.generaCortina();
                this.cortina = v_cortina.activa();

                this.campoReferencia.foco();

                return true;
            }

            async pideAlbaran(p_albaranesPedido) {
                if (!Array.isArray(p_albaranesPedido) || p_albaranesPedido.length === 0)
                    return 'nuevo';

                let v_albaranes = p_albaranesPedido
                    .map(p_albaran => ({
                        GGALBTIP_COD: p_albaran.GGALBTIP_COD,
                        N_ALBARAN: p_albaran.N_ALBARAN,
                        F_ALBARAN: p_albaran.F_ALBARAN,
                        SU_REF_ALB: p_albaran.SU_REF_ALB,
                        GGESTADO_COD: p_albaran.GGESTADO_COD
                    }));

                let v_resultadoSeleccionAlbaran = await dialogo()
                    .titulo('Albaranes')
                    .plantilla('dialogo-seleccion-albaran')
                    .construye(p_modal => new DialogoSeleccionAlbaran(p_modal, v_albaranes))
                    .abre();
                return v_resultadoSeleccionAlbaran;
            }

            // modifica el pedido en preparación (this.pedidoPreparado) indicando el albarán contra el que se sirvirá el pedido
            seleccionaAlbaran(p_resultadoSeleccionAlbaran) {
                let v_ggalbtip_cod, v_n_albaran, v_f_albaran /*, v_su_ref_alb */;
                if (p_resultadoSeleccionAlbaran !== "nuevo")
                    ({ GGALBTIP_COD: v_ggalbtip_cod, N_ALBARAN: v_n_albaran, F_ALBARAN: v_f_albaran, /* SU_REF_ALB: v_su_ref_alb */ } = p_resultadoSeleccionAlbaran);

                this.pedidoPreparado.GGALBTIP_COD = v_ggalbtip_cod;
                this.pedidoPreparado.N_ALBARAN = v_n_albaran;
                this.pedidoPreparado.F_ALBARAN = v_f_albaran;
                //this.pedidoPreparado.SU_REF_ALB = v_su_ref_alb;
            }

            async actualizaPreparadoArticuloSinLotesNiUbicaciones(p_valor) {
                let v_disponible = this.disponible[this.lineaSeleccionada.GAART_COD];

                await this.pideYActualizaBultos(this.lineaSeleccionada, v_disponible, { C_UNI1: this.lineaSeleccionada.C_PREP + p_valor, Detalles: [] });

                this.lineaSeleccionada = null;
                this.lecturaArticulo = null;
                this.lecturaCantidad = null;
                this.actualizaVista();
                this.campoReferencia.foco();
            }

            async pideYActualizaBultos(p_linea, p_disponible, p_preparado) {
                let v_c_uni1 = p_preparado.C_UNI1 - p_linea.C_PREP;

                let v_pedidoPreparado = this.pedidoPreparado;
                let v_pedido = this.pedido;
                let v_resultadoDialogoSeleccionBulto = await dialogo()
                    .titulo('Bultos')
                    .plantilla('dialogo-seleccion-bulto')
                    .construye(p_modal => new DialogoSeleccionBulto(p_modal, v_pedidoPreparado.Bultos, v_c_uni1))
                    .abre();
                if (typeof v_resultadoDialogoSeleccionBulto === 'undefined')
                    return;

                this.actualizaLinea(p_linea, p_disponible, p_preparado);

                switch (v_resultadoDialogoSeleccionBulto.opcion) {
                    case 'nuevo': {
                        let v_nuevoBulto =
                            Array.from(new Array(v_resultadoDialogoSeleccionBulto.numeroBultos), (_, p_indice) => ({
                                N_BULTO: v_pedidoPreparado.Bultos.length + p_indice + 1,
                                GVPALLETIP: v_resultadoDialogoSeleccionBulto.GVPALLETIP,
                                articulos: [{
                                    N_LIN: p_linea.N_LIN,
                                    GAART_COD: p_linea.GAART_COD,
                                    GAART_DESCRIP: p_linea.GAART_DESCRIP,
                                    CANTIDAD: v_resultadoDialogoSeleccionBulto.cantidadPorBulto,
                                }]
                            }))

                        v_pedidoPreparado.Bultos.push.apply(v_pedidoPreparado.Bultos, v_nuevoBulto);
                        break;
                    }
                    case 'actualiza': {
                        let v_bulto = v_pedidoPreparado.Bultos.find(p_bulto => p_bulto.N_BULTO === v_resultadoDialogoSeleccionBulto.bulto.N_BULTO);
                        if (typeof v_bulto === 'undefined')
                            throw new Error('No se ha encontrado el bulto a actualizar');

                        let v_linea = v_bulto.articulos.find(p => p.N_LIN === p_linea.N_LIN && p.GAART_COD === p_linea.GAART_COD);
                        if (typeof v_linea !== 'undefined')
                            v_linea.CANTIDAD += v_c_uni1;
                        else {
                            let v_nuevaLinea = {
                                N_LIN: p_linea.N_LIN,
                                GAART_COD: p_linea.GAART_COD,
                                GAART_DESCRIP: p_linea.GAART_DESCRIP,
                                CANTIDAD: v_c_uni1
                            };

                            v_bulto.articulos.push(v_nuevaLinea);
                        }
                        break;
                    }
                    case 'ninguno':
                        break;
                    default:
                        throw new Error('No es una opción válida');
                }
            }

            actualizaLinea(p_linea, p_disponible, p_preparado) {
                this.disponible[p_linea.GAART_COD] = p_disponible;

                let v_pedido = this.pedido;
                let v_lineaPedido = v_pedido.Lineas.find(p => p.N_LIN === p_linea.N_LIN);

                p_linea.C_PREP = p_preparado.C_UNI1;
                p_linea.C_PDTE = v_lineaPedido.C_PDTE - p_linea.C_PREP;
                p_linea.Detalles = p_preparado.Detalles;
            }

            async seleccionaLinea(p_lineaSeleccionada) {
                let v_lineaSeleccionada = p_lineaSeleccionada || null;

                this.lineaSeleccionada = v_lineaSeleccionada;

                this.lecturaArticulo = null;
                this.lecturaCantidad = null;

                if (v_lineaSeleccionada === null) {
                    this.actualizaVista();
                    this.campoReferencia.foco();
                    return;
                }
                try {
                    let v_disponible = this.disponible[v_lineaSeleccionada.GAART_COD];

                    if (v_lineaSeleccionada.GAART_LOTES_SN === 'S' || v_lineaSeleccionada.GAART_UBIC_SN === 'S') {
                        try {
                            let v_preparado = {
                                C_UNI1: v_lineaSeleccionada.C_UNI1,
                                Detalles: v_lineaSeleccionada.Detalles
                            };
                            let v_resultado = await dialogo()
                                .titulo('Lotes y ubicaciones')
                                .plantilla('dialogo-lotes-ubicaciones')
                                .construye(p_modal => new DialogoLotesUbicaciones(p_modal, v_lineaSeleccionada, v_disponible))
                                .abre();
                            if (typeof v_resultado === 'undefined')
                                return;

                            let { disponible: v_nuevoDisponible, preparado: v_nuevoPreparado } = v_resultado;
                            await this.pideYActualizaBultos(v_lineaSeleccionada, v_nuevoDisponible, v_nuevoPreparado);
                            this.campoReferencia.foco();
                        }
                        finally {
                            this.lineaSeleccionada = null;
                        }
                    } else {
                        this.lecturaArticulo = v_lineaSeleccionada.GAART_COD;
                        this.campoC_PREP.foco();
                    }
                }
                finally {
                    this.actualizaVista();
                }
            }

            sePuedeConfirmarPedido() {
                let v_sePuedeConfirmar = this.pedidoPreparado !== null && this.pedidoPreparado.Lineas.some(p_linea => p_linea.C_PREP !== 0);
                return v_sePuedeConfirmar;
            }

            actualizaPreparadoConResumenBultos(p_resumenBultos, p_opciones) {
                this.pedidoPreparado = Object.assign(this.pedidoPreparado, {
                    GGMENVIO_COD: p_resumenBultos.GGMENVIO_COD,
                    GGMENVIO_DES: p_resumenBultos.GGMENVIO_DES,
                    GGVEHICULO_COD: p_resumenBultos.GGVEHICULO_COD,
                    GGVEHICULO_DES: p_resumenBultos.GGVEHICULO_DES,

                    N_ETIQUETAS: p_resumenBultos.N_ETIQUETAS,
                    PESO: p_resumenBultos.PESO,
                    N_BULTOS: p_resumenBultos.N_BULTOS,

                    N_COPIAS: p_resumenBultos.N_COPIAS,
                    ImprimirValorado: p_resumenBultos.AlbaranValorado,
                    AlbaranValorado: p_resumenBultos.AlbaranValorado,
                    ImprimirEtiquetas: p_opciones.confirma && p_opciones.imprimeEtiquetas,
                    OPCION_ETIQUETAS: p_resumenBultos.OpcionPredeterminada,
                });
            }

            async confirmaPedido(p_opciones) {
                const v_copiaPedidoPreparado = _.cloneDeep(this.pedidoPreparado);

                let v_sinPreparar = this.pedidoPreparado.Lineas.every(p_linea => typeof p_linea.C_PREP === 'undefined');
                if (v_sinPreparar) {
                    await mensaje().error().muestra('Debe preparar el pedido antes de confirmarlo.');
                    this.pedidoPreparado = v_copiaPedidoPreparado;
                    return;
                }

                const { dejarPendiente: v_dejarPendiente } = p_opciones;

                this.pedidoPreparado = Object.assign(this.pedidoPreparado, {
                    DejarPendiente: v_dejarPendiente,
                });

                let v_resumenGeneraResumenBultos = await this.generaResumenBultos();
                if (typeof v_resumenGeneraResumenBultos === 'undefined') {
                    restauraPedidoPreparado();
                    this.pedidoPreparado = v_copiaPedidoPreparado;
                    return;
                }

                let v_resultadoPideResumenBultos = await this.pideResumenBultos(v_resumenGeneraResumenBultos, v_dejarPendiente);
                if (typeof v_resultadoPideResumenBultos === 'undefined') {
                    this.pedidoPreparado = v_copiaPedidoPreparado;
                    return;
                }

                //let [v_resumenBultos, v_opcion] = v_resultadoPideResumenBultos;
                //this.actualizaPreparadoConResumenBultos(v_resumenBultos, v_opcion);
                this.actualizaPreparadoConResumenBultos.apply(this, v_resultadoPideResumenBultos);

                if (v_dejarPendiente && this.pedidoPreparado.ImprimirEtiquetas) {
                    const v_respuestaPregunta = await pregunta()
                        .opciones(['Albarán + Etiquetas', 'Albarán'])
                        .opcionPredeterminada('Albarán')
                        .muestra("Se está dejando el albarán en estado PENDIENTE.\n ¿Desea imprimir etiquetas o solo el albarán?");
                    if (typeof v_respuestaPregunta !== 'undefined') {
                        let [v_indiceRespuestaPregunta] = v_respuestaPregunta;
                        if (v_indiceRespuestaPregunta === 1 /* Albarán */) {
                            this.pedidoPreparado = Object.assign(this.pedidoPreparado, {
                                ImprimirEtiquetas: false,
                            });
                        }
                    }
                }

                let v_resultadoGeneraAlbaran = await this.generaAlbaran();
                if (typeof v_resultadoGeneraAlbaran === 'undefined') {
                    this.pedidoPreparado = v_copiaPedidoPreparado;
                    return;
                }

                if (typeof v_resultadoGeneraAlbaran.PREGUNTA_HIST === 'string' && v_resultadoGeneraAlbaran.PREGUNTA_HIST.length > 0) {
                    const v_respuestaPregunta = await pregunta()
                        .cancelable(false)
                        .muestra(v_resultadoGeneraAlbaran.PREGUNTA_HIST);

                    if (typeof v_respuestaPregunta !== 'undefined') {
                        const
                            v_urls = ['/Expedicion/PonerHistorico', '/Expedicion/DejarActivo'],
                            [v_dejarHistorico] = v_respuestaPregunta,
                            v_url = v_urls[v_dejarHistorico];

                        await api(v_url)
                            .metodo('post')
                            .parametros({
                                EMPGRUPO_COD: this.pedidoPreparado.EMPGRUPO_COD,
                                EMP_COD: this.pedidoPreparado.EMP_COD,
                                GGPEDTIP_COD: this.pedidoPreparado.GGPEDTIP_COD,
                                N_PEDIDO: this.pedidoPreparado.N_PEDIDO
                            })
                            .invoca();
                    }
                }

                notificacion().muestra(`Se ha generado el albarán ${v_resultadoGeneraAlbaran.GGALBTIP_COD}/${v_resultadoGeneraAlbaran.N_ALBARAN}`);

                this.inicializa();
            }

            async generaResumenBultos() {
                let v_resumenBultos;
                try {
                    [v_resumenBultos] = await api('/Expedicion/ResumenBultos')
                        .metodo('put')
                        .parametros({
                            EMPGRUPO_COD: this.pedidoPreparado.EMPGRUPO_COD,
                            EMP_COD: this.pedidoPreparado.EMP_COD,
                            GGPEDTIP_COD: this.pedidoPreparado.GGPEDTIP_COD,
                            N_PEDIDO: this.pedidoPreparado.N_PEDIDO,
                            GGALBTIP_COD: this.pedidoPreparado.GGALBTIP_COD,
                            N_ALBARAN: this.pedidoPreparado.N_ALBARAN,
                            GGMENVIO_COD: this.pedidoPreparado.GGMENVIO_COD,
                            GGVEHICULO_COD: this.pedidoPreparado.GGVEHICULO_COD,
                            N_BULTOS: this.pedidoPreparado.N_BULTOS,
                            Bultos: this.pedidoPreparado.Bultos,
                            PESO: this.pedidoPreparado.PESO,
                            N_ETIQUETAS: this.pedidoPreparado.N_ETIQUETAS,
                            Lineas: this.pedidoPreparado.Lineas
                                .filter(p_linea => p_linea.C_PREP !== 0)
                                .map(p_linea => ({
                                    N_LIN: p_linea.N_LIN,
                                    C_UNI1: p_linea.C_UNI1,
                                    //C_UNI2: p_linea.C_UNI2,
                                    Detalles: p_linea.Detalles
                                        .filter(p_detalle => p_detalle.C_UNI1 !== null)
                                        .map(p_detalle => ({
                                            GAUBIC_COD: p_detalle.GAUBIC_COD,
                                            LOTE: p_detalle.LOTE,
                                            C_UNI1: p_detalle.C_UNI1
                                        }))
                                })),
                            N_COPIAS: this.pedidoPreparado.N_COPIAS,
                            DejarPendiente: this.pedidoPreparado.DejarPendiente,
                            ImprimirValorado: this.pedidoPreparado.ImprimirValorado,
                            AlbaranValorado: this.pedidoPreparado.AlbaranValorado,
                            OPCION_ETIQUETAS: this.pedidoPreparado.OPCION_ETIQUETAS,
                            ImprimirEtiquetas: this.pedidoPreparado.ImprimirEtiquetas,
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje().error().muestra(v_mensaje);
                    return;
                }
                return v_resumenBultos;
            }

            async pideResumenBultos(p_resumenBultos, p_dejarPendiente) {
                let v_resultadoPideResumenBultos = await dialogo()
                    .titulo('Datos de envío')
                    .plantilla('dialogo-resumen-bultos')
                    .construye(p_modal =>
                        new DialogoResumenBultos(p_modal,
                            Object.assign(p_resumenBultos, {
                                GGMENVIO_COD: this.pedidoPreparado.GGMENVIO_COD,
                                GGMENVIO_DES: this.pedidoPreparado.GGMENVIO_DES,
                                GGVEHICULO_COD: this.pedidoPreparado.GGVEHICULO_COD,
                                GGVEHICULO_DES: this.pedidoPreparado.GGVEHICULO_DES,
                                AlbaranValorado: this.pedidoPreparado.AlbaranValorado,
                            }), {
                            mostrarAlbaranValorado: this.pedido.MostrarAlbaranValorado,
                            mostrarCopias: p_dejarPendiente,
                            detalleBultos: this.pedidoPreparado.Bultos
                        }))
                    .abre();
                return v_resultadoPideResumenBultos;
            }

            async generaAlbaran() {
                let v_resultadoExpedir;
                try {
                    [v_resultadoExpedir] = await api('/Expedicion/Expedir')
                        .metodo('post')
                        .parametros({
                            EMPGRUPO_COD: this.pedidoPreparado.EMPGRUPO_COD,
                            EMP_COD: this.pedidoPreparado.EMP_COD,
                            GGPEDTIP_COD: this.pedidoPreparado.GGPEDTIP_COD,
                            N_PEDIDO: this.pedidoPreparado.N_PEDIDO,
                            GGALBTIP_COD: this.pedidoPreparado.GGALBTIP_COD,
                            N_ALBARAN: this.pedidoPreparado.N_ALBARAN,
                            F_ALBARAN: moment.utc(this.pedidoPreparado.F_ALBARAN, 'DD/MM/YYYY').toISOString(true),
                                // moment(this.pedidoPreparado.F_ALBARAN, 'DD/MM/YYYY').utc().toISOString(true),
                                
                            GGMENVIO_COD: this.pedidoPreparado.GGMENVIO_COD,
                            GGVEHICULO_COD: this.pedidoPreparado.GGVEHICULO_COD,
                            N_BULTOS: this.pedidoPreparado.N_BULTOS,
                            Bultos: this.pedidoPreparado.Bultos
                                .reduce((p_nuevosBultos, p_bulto) => {
                                    p_nuevosBultos = p_nuevosBultos.concat(p_bulto.articulos.map(p_articulo => ({
                                        N_BULTO: p_bulto.N_BULTO,
                                        GAART_COD: p_articulo.GAART_COD,
                                        GAART_DESCRIP: p_articulo.GAART_DESCRIP,
                                        C_UNI1: p_articulo.CANTIDAD,
                                        GGPEDLIN_N_LIN: p_articulo.N_LIN,
                                        GVPALLETIP_COD: p_bulto.GVPALLETIP.COD,
                                    })));
                                    return p_nuevosBultos;
                                }, []),
                            PESO: this.pedidoPreparado.PESO,
                            N_ETIQUETAS: this.pedidoPreparado.N_ETIQUETAS,
                            Lineas: this.pedidoPreparado.Lineas
                                .filter(p_linea => typeof p_linea.C_PREP !== 'undefined')
                                .map(p_linea => ({
                                    N_LIN: p_linea.N_LIN,
                                    C_UNI1: p_linea.C_PREP,
                                    //C_UNI2: p_linea.C_UNI2,
                                    Detalles: p_linea.Detalles
                                        .filter(p_detalle => p_detalle.C_UNI1 !== null)
                                        .map(p_detalle => ({
                                            GAUBIC_COD: p_detalle.GAUBIC_COD,
                                            LOTE: p_detalle.LOTE,
                                            C_UNI1: p_detalle.C_UNI1
                                        }))
                                })),
                            ImprimirEtiquetas: this.pedidoPreparado.ImprimirEtiquetas,
                            OPCION_ETIQUETAS: this.pedidoPreparado.OpcionPredeterminada,
                            ImprimirValorado: this.pedidoPreparado.AlbaranValorado,
                            DejarPendiente: this.pedidoPreparado.DejarPendiente,
                            N_COPIAS: this.pedidoPreparado.N_COPIAS
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje().error().muestra(v_mensaje);
                    return;
                }
                return v_resultadoExpedir;
            }

            async obtenerLinea() {
                const v_lecturaArticulo = this.lecturaArticulo;

                // Buscamos el código de artículo en las lineas del pedido
                const v_linea = this.tablaLineas.buscaYSelecciona({ GAART_COD: v_lecturaArticulo }, { seleccionaPrimero: true });
                if (typeof v_linea !== 'undefined')
                    return;

                // Si no lo hemos encontrado, comprobamos si lo que se ha leído es una codificación alternativa
                let v_articulos;
                try {
                    [v_articulos] = await api('/Expedicion/Articulos')
                        .parametros({
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            codigoBarras: v_lecturaArticulo
                        })
                        .invoca();
                }
                catch ([v_mensaje]) {
                    await mensaje().error().muestra(v_mensaje);
                    return;
                }

                v_articulos = v_articulos.filter(p_articulo => this.pedido.Lineas.find(p_linea => p_linea.GAART_COD === p_articulo.GAART_COD));

                let v_gaart_cod;
                // Se ha encontrado un único artículo con la codificación alternativa leíd
                if (v_articulos.length === 1) {
                    // Se ha encontrado un único artículo con la codificación alternativa leíd
                    v_gaart_cod = v_articulos[0].GAART_COD;
                }
                else if (v_articulos.length > 1) {
                    // Se han encontrado varios articulos, por lo que daremos a elegir uno al usuario
                    const v_articulo = await lov()
                        .titulo('Lista de artículos')
                        .columnas([
                            { propiedad: 'GAART_COD', titulo: 'Código', ancho: '6em' },
                            { propiedad: 'GAART_DESCRIP', titulo: 'Nombre' }
                        ])
                        .elige(v_articulos);
                    if (typeof v_articulo !== 'undefined')
                        v_gaart_cod = v_articulo.GAART_COD;
                }

                // si ya tenemos artículo comprobamos que esté en el pedido
                if (v_gaart_cod) {
                    const v_linea = this.tablaLineas.buscaYSelecciona({ GAART_COD: v_gaart_cod }, { seleccionaPrimero: true });
                    if (typeof v_linea !== 'undefined')
                        return;
                }

                // al final, si llegamos a este punto, es que no tenemos un artículo válido
                await notificacion().error().muestra('Referencia erronea o inexistente.');
                this.lecturaArticulo = null;
                this.lecturaCantidad = null;
                this.campoReferencia.foco();
            }
        }

        return PantallaExpedicion;
    });